#! /usr/bin/python3.6
# module initially auto generated using V5Automation.chm from CATIA V5 R25

from pycatia.system_interfaces.any_object import AnyObject
from pycatia.knowledge_interfaces.angle import Angle
from pycatia.in_interfaces.reference import Reference
from pycatia.enumeration.enumeration_types import cat_axis_system_axis_type


class AxisSystem(AnyObject):
    """
        .. note::
            CAA V5 Visual Basic help

                | The object Axis System  A axis system has got one origin point and
                | three orthogonal axes,  crossing at the origin point.

    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.axis_system = com_object

    @property
    def axis_rotation_angle(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | AxisRotationAngle
                | o Property AxisRotationAngle() As Angle
                | 
                | Returns the rotation angle of an axis system. Succeeds only if the
                | axis system is defined by a rotation around an axis, which means that
                | its type is catAxisSystemAxisRotation.

        """
        return Angle(self.axis_system.AxisRotationAngle)

    @property
    def axis_rotation_reference(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | AxisRotationReference
                | o Property AxisRotationReference() As Reference
                | 
                | Returns the reference for the axis rotation. Succeeds only if the axis
                | system is defined by a rotation around an axis, which means that its
                | type is catAxisSystemAxisRotation.

        """
        return Reference(self.axis_system.AxisRotationReference)

    @property
    def is_current(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | IsCurrent
                | o Property IsCurrent() As boolean
                | 
                | Returns True if the axis system is the current one, else returns
                | False.  Sets the axis system as the current one or not.  Example:The
                | following example returns in isCurrent True if the axis system
                | axisSystem is the current one :   isCurrent = axisSystem.IsCurrentThe
                | following example sets the axis system axisSystem as the current one :
                | axisSystem.IsCurrent = 1The following example sets the axis system
                | axisSystem as not the current one :   axisSystem.IsCurrent = 0

        """
        return self.axis_system.IsCurrent

    @property
    def origin_point(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | OriginPoint
                | o Property OriginPoint() As Reference
                | 
                | Returns or sets the geometric point which defines the origin of the
                | axis system.   OriginPoint is and must be a reference on a geometric
                | 3D point.  Example:The following example sets the point Point.1 of the
                | Geometrical Set  Geometrical Set.1 as the origin point of the axis
                | system AxisSystem0:   Dim HybridBody4 As AnyObject Set HybridBody4 =
                | Body1.HybridBodies.Item  ( "Geometrical Set.1" )  Dim
                | HybridShapePointCoord5 As AnyObject Set HybridShapePointCoord5 =
                | HybridBody4.HybridShapes.Item  ( "Point.1" )  Dim Reference6 As
                | Reference Set Reference6 =
                | CATIA.ActiveDocument.Part.CreateReferenceFromGeometry  (
                | HybridShapePointCoord5 )  AxisSystem0.OriginPoint = Reference6

        """
        return Reference(self.axis_system.OriginPoint)

    @property
    def origin_type(self):
        """

        .. note::
            CAA V5 Visual Basic help

                | OriginType
                | o Property OriginType() As CATAxisSystemOriginType
                | 
                | Returns or sets the way the origin point is defined.  The origin point
                | can be not specified, or be defined by coordinates or by a geometric
                | point. CATAxisSystemOriginType is the enumeration which describes how
                | the origin point is defined : If OriginType=0, the origin point is
                | defined by a geometric point. If no point si selected, the origin will
                | be automatically put at the intersection of the lines or planes
                | defining the axes.  If OriginType=1, the origin is defined by three
                | coordinates x,y,z. Then, the origin will always stays at the position
                | defined by the coordinates.  Example:The following example prints the
                | origin type :  Catia.SystemService.Print " OriginType = " &
                | axisSystem.OriginTypeThe following example sets the origin type to 1 :
                | axisSystem.OriginType = 1

            :return int:
        """
        return self.axis_system.OriginType

    @origin_type.setter
    def origin_type(self, value):
        types = [x for x in cat_axis_system_axis_type]
        if not isinstance(value, int) or value not in types:
            raise ValueError(
                f'Value "value" must be an int and in the range "{types}".'
            )

    @property
    def type(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Type
                | o Property Type() As CATAxisSystemMainType
                | 
                | Returns or sets the type of the axis system.  Sets the axis system
                | type.  Example:The following example returns in type1 the type of the
                | axis system axisSystem1:   type1 = axisSystem1.TypeThe following
                | example sets the type of the axis system axisSystem1 as standard:
                | axisSystem1.Type = 0The following example sets the type of the axis
                | system axisSystem1 as axis rotation:   axisSystem1.Type = 1The
                | following example sets the type of the axis system axisSystem1 as
                | datum (explicit):   axisSystem1.Type = 3

        """
        return self.axis_system.Type

    @type.setter
    def type(self, value):
        types = [x for x in cat_axis_system_axis_type]
        if not isinstance(value, int) or value not in types:
            raise ValueError(
                f'Value "value" must be an int and in the range "{types}".'
            )

    @property
    def x_axis_direction(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | XAxisDirection
                | o Property XAxisDirection() As Reference
                | 
                | Reads or sets the geometric point, line or plane which defines the
                | direction of the X axis. AxisDirection is and must be a reference on a
                | 3D point or 3D line or plane.  Example:The following example sets the
                | point Point.1 of the Geometrical Set  Geometrical Set.1 as the
                | direction of the X axis of the axis system AxisSystem0:  Dim
                | HybridBody4 As AnyObject Set HybridBody4 = Body1.HybridBodies.Item  (
                | "Geometrical Set.1" )  Dim HybridShapePointCoord5 As AnyObject Set
                | HybridShapePointCoord5 = HybridBody4.HybridShapes.Item  ( "Point.1" )
                | Dim Reference6 As Reference Set Reference6 =
                | CATIA.ActiveDocument.Part.CreateReferenceFromGeometry  (
                | HybridShapePointCoord5 )  AxisSystem0.XAxisDirection = Reference6

        :return: int
        """
        return self.axis_system.XAxisDirection

    @x_axis_direction.setter
    def x_axis_direction(self, value):
        types = [x for x in cat_axis_system_axis_type]
        if not isinstance(value, int) or value not in types:
            raise ValueError(
                f'Value "value" must be an int and in the range "{types}".'
            )

    @property
    def x_axis_type(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | XAxisType
                | o Property XAxisType() As CATAxisSystemAxisType
                | 
                | Reads or sets the way the X axis is specified.  An axis X,Y, or Z of
                | the axis system can be defined  by a geometric point, line or plane,
                | or by coordinates.  AxisType = 0 : The axis is defined by a geometric
                | point, line or plane and with the same direction.         AxisType = 1
                | : The axis direction is defined by the three coordinates x,y,z, of a
                | vector, to which the axis will always stay parallel.    AxisType = 2
                | : the axis is defined by a geometric point, line or plane and with the
                | opposite direction. Notice : If the X axis is neither defined by
                | coordinates nor by a point,line or plane, the axis will be
                | automatically computed in order to build an orthogonal  axis system
                | with the other specified axes.  Example:The following example prints
                | the X axis type :  Catia.SystemService.Print " XAxisType = " &
                | axisSystem.XAxisTypeThe following example sets the X axis type to 1 :
                | axisSystem.XAxisType = 1

        :return: Reference()
        """
        return Reference(self.axis_system.XAxisType)

    @property
    def y_axis_direction(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | YAxisDirection
                | o Property YAxisDirection() As Reference
                | 
                | Reads or sets the geometric point, line or plane which defines the
                | direction of the Y axis. AxisDirection is and must be a reference on a
                | 3D point or 3D line or plane.  Example:The following example sets the
                | point Point.1 of the Geometrical Set Geometrical Set.1 as the
                | direction of the Y axis of the axis system AxisSystem0:  Dim
                | HybridBody4 As AnyObject Set HybridBody4 = Body1.HybridBodies.Item  (
                | "Geometrical Set.1" )  Dim HybridShapePointCoord5 As AnyObject Set
                | HybridShapePointCoord5 = HybridBody4.HybridShapes.Item  ( "Point.1" )
                | Dim Reference6 As Reference Set Reference6 =
                | CATIA.ActiveDocument.Part.CreateReferenceFromGeometry  (
                | HybridShapePointCoord5 )  AxisSystem0.YAxisDirection = Reference6

        :return: int
        """
        return self.axis_system.YAxisDirection

    @y_axis_direction.setter
    def y_axis_direction(self, value):
        types = [x for x in cat_axis_system_axis_type]
        if not isinstance(value, int) or value not in types:
            raise ValueError(
                f'Value "value" must be an int and in the range "{types}".'
            )

    @property
    def y_axis_type(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | YAxisType
                | o Property YAxisType() As CATAxisSystemAxisType
                | 
                | Reads or sets the way the Y axis is specified.  An axis X,Y, or Z of
                | the axis system can be defined  by a geometric point, line or plane,
                | or by coordinates.  AxisType = 0 : The axis is defined by a geometric
                | point, line or plane and with the same direction.         AxisType = 1
                | : The axis direction is defined by the three coordinates x,y,z, of a
                | vector, to which the axis will always stay parallel.    AxisType = 2
                | : the axis is defined by a geometric point, line or plane and with the
                | opposite direction. Notice : If the Y axis is neither defined by
                | coordinates nor by a point,line or plane, the axis will be
                | automatically computed in order to build an orthogonal  axis system
                | with the other specified axes.  Example:The following example prints
                | the Y axis type :  Catia.SystemService.Print " YAxisType = " &
                | axisSystem.YAxisTypeThe following example sets the Y axis type to 1 :
                | axisSystem.YAxisType = 1

        :return: Reference()
        """
        return Reference(self.axis_system.YAxisType)

    @property
    def z_axis_direction(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | ZAxisDirection
                | o Property ZAxisDirection() As Reference
                | 
                | Reads or sets the geometric point, line or plane which defines the
                | direction of the Z axis. AxisDirection is and must be a reference on a
                | 3D point or 3D line or plane.  Example:The following example sets the
                | point Point.1 of the Geometrical Set Geometrical Set.1 as the
                | direction of the Z axis of the axis system AxisSystem0:  Dim
                | HybridBody4 As AnyObject Set HybridBody4 = Body1.HybridBodies.Item  (
                | "Geometrical Set.1" )  Dim HybridShapePointCoord5 As AnyObject Set
                | HybridShapePointCoord5 = HybridBody4.HybridShapes.Item  ( "Point.1" )
                | Dim Reference6 As Reference Set Reference6 =
                | CATIA.ActiveDocument.Part.CreateReferenceFromGeometry  (
                | HybridShapePointCoord5 )  AxisSystem0.ZAxisDirection = Reference6

        :return: int
        """
        return self.axis_system.ZAxisDirection

    @y_axis_direction.setter
    def y_axis_direction(self, value):
        types = [x for x in cat_axis_system_axis_type]
        if not isinstance(value, int) or value not in types:
            raise ValueError(
                f'Value "value" must be an int and in the range "{types}".'
            )

    @property
    def z_axis_type(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | ZAxisType
                | o Property ZAxisType() As CATAxisSystemAxisType
                | 
                | Reads or sets the way the Z axis is specified.  An axis X,Y, or Z of
                | the axis system can be defined  by a geometric point, line or plane,
                | or by coordinates.  AxisType = 0 : The axis is defined by a geometric
                | point, line or plane and with the same direction.         AxisType = 1
                | : The axis direction is defined by the three coordinates x,y,z, of a
                | vector, to which the axis will always stay parallel.    AxisType = 2
                | : the axis is defined by a geometric point, line or plane and with the
                | opposite direction. Notice : If the Z axis is neither defined by
                | coordinates nor by a point,line or plane, the axis will be
                | automatically computed in order to build an orthogonal  axis system
                | with the other specified axes.  Example:The following example prints
                | the Z axis type :  Catia.SystemService.Print " ZAxisType = " &
                | axisSystem.ZAxisTypeThe following example sets the Z axis type to 1 :
                | axisSystem.ZAxisType = 1

        :return: Reference()
        """
        return Reference(self.axis_system.ZAxisType)

    def get_euler_angles(self, o_first_angle, o_second_angle, third_angle):
        """
        .. note::
            CAA V5 Visual Basic help

                | GetEulerAngles
                | o Sub GetEulerAngles(    Angle    oFirstAngle,
                |                          Angle    oSecondAngle,
                |                          Angle    ThirdAngle)
                | 
                | Returns the Euler Angles of an axis system. Succeeds only if the axis
                | system is defined by Euler angles, which means its type is
                | catAxisSystemEulerAngles.

        """
        return self.axis_system.GetEulerAngles(
            o_first_angle, o_second_angle, third_angle
        )

    def get_origin(self, o_origin):
        """
        .. note::
            CAA V5 Visual Basic help

                | GetOrigin
                | o Sub GetOrigin(    CATSafeArrayVariant    oOrigin)
                | 
                | Returns the coordinates X,Y,Z of the origin point of the axis system.
                |
                | Parameters:
                | oOrigin
                |    A Safe Array made up of 3 doubles: X, Y, Z, representing the 
                |    coordinates in model space of the origin point of the axis system.
                |
                | Examples:
                | 
                | The following example retrieves in originCoord the coordinates
                | of the origin point of the axisSystem axis system:
                | 
                | Dim originCoord(2)
                | axisSystem.GetOrigin originCoord

        """
        return self.axis_system.GetOrigin(o_origin)

    def get_vectors(self, o_vector_x, o_vector_y):
        """
        .. note::
            CAA V5 Visual Basic help

                | GetVectors
                | o Sub GetVectors(    CATSafeArrayVariant    oVectorX,
                |                      CATSafeArrayVariant    oVectorY)
                | 
                | Returns the coordinates X,Y,Z of the axes X and Y of the axis system.
                |
                | Parameters:
                | oVectorX
                |    A Safe Array made up of 3 doubles: X, Y, Z, representing the 
                |    coordinates in model space of the X axis vector of the axis system.
                |  
                |  oVectorY
                |    A Safe Array made up of 3 doubles: X, Y, Z, representing the 
                |    coordinates in model space of the Y axis vector of the axis system.
                |
                | Examples:
                | 
                | The following example retrieves in vectorXCoord and vectorYCoord
                | the coordinates of the vectors of the axisSystem axis system:
                | 
                | Dim vectorXCoord(2)
                | Dim vectorYCoord(2)
                | axisSystem.GetVectors vectorXCoord, vectorYCoord


        """
        return self.axis_system.GetVectors(o_vector_x, o_vector_y)

    def get_x_axis(self, o_x_axis):
        """
        .. note::
            CAA V5 Visual Basic help

                | GetXAxis
                | o Sub GetXAxis(    CATSafeArrayVariant    oXAxis)
                | 
                | Returns the coordinates X,Y,Z of the X axis of the axis system.
                |
                | Parameters:
                | oXAxis
                |    A Safe Array made up of 3 doubles: X, Y, Z, representing the 
                |    coordinates in model space of the X axis of the axis system.
                |
                | Examples:
                | 
                | The following example retrieves in XAxisCoord the coordinates
                | of the X axis of the axisSystem axis system:
                | 
                | Dim XAxisCoord(2)
                | axisSystem.GetXAxis XAxisCoord


        """
        return self.axis_system.GetXAxis(o_x_axis)

    def get_y_axis(self, o_y_axis):
        """
        .. note::
            CAA V5 Visual Basic help

                | GetYAxis
                | o Sub GetYAxis(    CATSafeArrayVariant    oYAxis)
                | 
                | Returns the coordinates X,Y,Z of the Y axis of the axis system.
                |
                | Parameters:
                | oYAxis
                |    A Safe Array made up of 3 doubles: X, Y, Z, representing the 
                |    coordinates in model space of the Y axis of the axis system.
                |
                | Examples:
                | 
                | The following example retrieves in YAxisCoord the coordinates
                | of the Y axis of the axisSystem axis system:
                | 
                | Dim YAxisCoord(2)
                | axisSystem.GetYAxis XAxisCoord


        """
        return self.axis_system.GetYAxis(o_y_axis)

    def get_z_axis(self, o_z_axis):
        """
        .. note::
            CAA V5 Visual Basic help

                | GetZAxis
                | o Sub GetZAxis(    CATSafeArrayVariant    oZAxis)
                | 
                | Returns the coordinates X,Y,Z of the Z axis of the axis system.
                |
                | Parameters:
                | oZAxis
                |    A Safe Array made up of 3 doubles: X, Y, Z, representing the 
                |    coordinates in model space of the Z axis of the axis system.
                |
                | Examples:
                | 
                | The following example retrieves in ZAxisCoord the coordinates
                | of the Z axis of the axisSystem axis system:
                | 
                | Dim ZAxisCoord(2)
                | axisSystem.GetZAxis ZAxisCoord


        """
        return self.axis_system.GetZAxis(o_z_axis)

    def put_origin(self, i_origin):
        """
        .. note::
            CAA V5 Visual Basic help

                | PutOrigin
                | o Sub PutOrigin(    CATSafeArrayVariant    iOrigin)
                | 
                | Defines the coordinates X,Y,Z of the origin point of the axis system.
                |
                | Parameters:
                | iOrigin
                |    A Safe Array made up of 3 doubles: X, Y, Z, representing the 
                |    coordinates in model space of the origin point of the axis system.
                |
                | Examples:
                | 
                | The following example puts in originCoord the new coordinates of
                | the origin point of the axisSystem axis system:
                | 
                | Dim originCoord(2)
                | originCoord ( 0 )  = 100.000000
                | originCoord ( 1 )  = 200.000000
                | originCoord ( 2 )  = 10.000000
                | axisSystem.PutOrigin originCoord


        """
        return self.axis_system.PutOrigin(i_origin)

    def put_vectors(self, i_vector_x, i_vector_y):
        """
        .. note::
            CAA V5 Visual Basic help

                | PutVectors
                | o Sub PutVectors(    CATSafeArrayVariant    iVectorX,
                |                      CATSafeArrayVariant    iVectorY)
                | 
                | Defines the coordinates X,Y,Z of the axes X and Y of the axis system.
                |
                | Parameters:
                | iVectorX
                |    A Safe Array made up of 3 doubles: X, Y, Z, representing the 
                |    coordinates in model space of the X axis vector of the axis system.
                |  iVectorY
                |    A Safe Array made up of 3 doubles: X, Y, Z, representing the 
                |    coordinates in model space of the Y axis vector of the axis system.
                |
                | Examples:
                | 
                | The following example modifies in vectorXCoord and vectorYCoord
                | the coordinates of the vectors of the axisSystem axis system:
                | 
                | Dim vectorXCoord(2)
                | vectorYCoord ( 0 )  = 1.000000
                | vectorYCoord ( 1 )  = -1.000000
                | vectorYCoord ( 2 )  = 0.000000
                | Dim vectorYCoord(2)
                | vectorYCoord ( 0 )  = 0.000000
                | vectorYCoord ( 1 )  = 0.000000
                | vectorYCoord ( 2 )  = 1.000000
                | axisSystem.PutVectors vectorXCoord, vectorYCoord


        """
        return self.axis_system.PutVectors(i_vector_x, i_vector_y)

    def put_x_axis(self, i_x_axis):
        """
        .. note::
            CAA V5 Visual Basic help

                | PutXAxis
                | o Sub PutXAxis(    CATSafeArrayVariant    iXAxis)
                | 
                | Defines the coordinates X,Y,Z of the X axis of the axis system.
                |
                | Parameters:
                | iXAxis
                |    A Safe Array made up of 3 doubles: X, Y, Z, representing the 
                |    coordinates in model space of the X axis of the axis system.
                |
                | Examples:
                | 
                | The following example puts in XAxisCoord the new coordinates of
                | the X axis of the axisSystem axis system:
                | 
                | Dim XAxis(2)
                | XAxis ( 0 )  = 100.000000
                | XAxis ( 1 )  = 200.000000
                | XAxis ( 2 )  = 10.000000
                | axisSystem.PutXAxis XAxis


        """
        return self.axis_system.PutXAxis(i_x_axis)

    def put_y_axis(self, i_y_axis):
        """
        .. note::
            CAA V5 Visual Basic help

                | PutYAxis
                | o Sub PutYAxis(    CATSafeArrayVariant    iYAxis)
                | 
                | Defines the coordinates X,Y,Z of the Y axis of the axis system.
                |
                | Parameters:
                | iYAxis
                |    A Safe Array made up of 3 doubles: X, Y, Z, representing the 
                |    coordinates in model space of the Y axis of the axis system.
                |
                | Examples:
                | 
                | The following example puts in XAxisCoord the new coordinates of
                | the Y axis of the axisSystem axis system:
                | 
                | Dim YAxis(2)
                | YAxis ( 0 )  = 100.000000
                | YAxis ( 1 )  = 200.000000
                | YAxis ( 2 )  = 10.000000
                | axisSystem.PutYAxis YAxis


        """
        return self.axis_system.PutYAxis(i_y_axis)

    def put_z_axis(self, i_z_axis):
        """
        .. note::
            CAA V5 Visual Basic help

                | PutZAxis
                | o Sub PutZAxis(    CATSafeArrayVariant    iZAxis)
                | 
                | Defines the coordinates X,Y,Z of the Z axis of the axis system.
                |
                | Parameters:
                | iZAxis
                |    A Safe Array made up of 3 doubles: X, Y, Z, representing the 
                |    coordinates in model space of the Z axis of the axis system.
                |
                | Examples:
                | 
                | The following example puts in ZAxisCoord the new coordinates of
                | the Z axis of the axisSystem axis system:
                | 
                | Dim ZAxis(2)
                | ZAxis ( 0 )  = 100.000000
                | ZAxis ( 1 )  = 200.000000
                | ZAxis ( 2 )  = 10.000000
                | axisSystem.PutZAxis ZAxis


        """
        return self.axis_system.PutZAxis(i_z_axis)

    def __repr__(self):
        return f"AxisSystem()"



#! /usr/bin/python3.6
# module initially auto generated using V5Automation.chm from CATIA R25 on 2020-05-18 10:56:40.651039

from pycatia.system_interfaces.any_object import AnyObject
from .measurable import Measurable


class SPAWorkbench(AnyObject):
    """
        .. note::
            CAA V5 Visual Basic help

                | The object to manage all Space Analysis objects.This version allows
                | you to manage inertia data, clashes, distances and sections.

    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.spa_workbench = com_object.GetWorkbench("SPAWorkbench")

    @property
    def clashes(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Clashes
                | o Property Clashes() As   (Read Only)
                | 
                | Returns the Clashes collection. WARNING: this method will be
                | DEPRECATED in the next release. It is recommended to use the
                | method GetTechnologicalObject("Clashes") on the root
                | product, to retrieve the Clashes collection.
                |
                | Example: This example retrieves the Clashes collection of the active
                | document.
                | Dim TheSPAWorkbench As Workbench
                | Set TheSPAWorkbench = CATIA.ActiveDocument.GetWorkbench ("SPAWorkbench")
                | Dim TheClashesList As Clashes
                | Set TheClashesList = TheSPAWorkbench.Clashes

        :return:
        """
        return self.spa_workbench.Clashes

    @property
    def distances(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Distances
                | o Property Distances() As   (Read Only)
                | 
                | Returns the Distances collection. WARNING: this method will
                | be DEPRECATED in the next release. It is recommended to use
                | the method GetTechnologicalObject("Distances") on the root
                | product, to retrieve the Distances collection.
                |
                | Example: This example retrieves the Distances collection of
                | the active document.
                | Dim TheSPAWorkbench As Workbench
                | Set TheSPAWorkbench = CATIA.ActiveDocument.GetWorkbench ("SPAWorkbench" )
                | Dim TheDistancesList As Distances
                | Set TheDistancesList = TheSPAWorkbench.Distances

        :return:
        """
        return self.spa_workbench.Distances

    @property
    def inertias(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Inertias
                | o Property Inertias() As   (Read Only)
                | 
                | Returns the Inertias collection. WARNING: this method will
                | be DEPRECATED in the next release. It is recommended to use
                | the method GetTechnologicalObject("Inertia") on the product
                | to analyze, to retrieve an Inertia object.
                |
                | Example: This
                | example retrieves the Inertias collection of the active
                | document.
                | Dim TheSPAWorkbench As Workbench
                | Set TheSPAWorkbench = CATIA.ActiveDocument.GetWorkbench ( "SPAWorkbench" )
                | Dim TheInertiasList As Inertias
                | Set TheInertiasList = TheSPAWorkbench.Inertias
                |

        :return:
        """
        return self.spa_workbench.Inertias

    @property
    def sections(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Sections
                | o Property Sections() As   (Read Only)
                | 
                | Returns the Sections collection. WARNING: this method will
                | be DEPRECATED in the next release. It is recommended to use
                | the method GetTechnologicalObject("Sections") on the root
                | product, to retrieve the Sections collection.
                |
                | Example: This example retrieves the Sections collection of the active
                | document.
                | Dim TheSPAWorkbench As Workbench
                | Set TheSPAWorkbench = CATIA.ActiveDocument.GetWorkbench ( "SPAWorkbench" )
                | Dim TheSectionsList As Sections
                | Set TheSectionsList = TheSPAWorkbench.Sections

        :return:
        """
        return self.spa_workbench.Sections

    def get_measurable(self, i_measured_item):
        """
        .. note::
            CAA V5 Visual Basic help

                | GetMeasurable
                | o Func GetMeasurable(iMeasuredItem) As
                | 
                | Returns the Measurable object.
                |
                | Example: This example get the Measurable from the SPAWorkBench.
                | Dim referenceObject As referenceObject
                | Set referenceObject = "GetReference"
                | Dim TheSPAWorkbench As Workbench
                | Set TheSPAWorkbench = CATIA.ActiveDocument.GetWorkbench ( "SPAWorkbench" )
                | Dim TheMeasurable As Measurable
                | Set TheMeasurable = TheSPAWorkbench.GetMeasurable(referenceObject)

        :param i_measured_item:
        :return: Measurable()
        """
        return Measurable(self.spa_workbench.GetMeasurable(i_measured_item.com_object))

    def __repr__(self):
        return f'SPAWorkbench()'



#! /usr/bin/python3.6
# module initially auto generated using V5Automation.chm from CATIA V5 R25

from pathlib import Path

from pycatia.hybrid_shape_interfaces.hybrid_shape_factory import HybridShapeFactory
from pycatia.in_interfaces.reference import Reference
from pycatia.knowledge_interfaces.parameters import Parameters
from pycatia.knowledge_interfaces.relations import Relations
from pycatia.part_interfaces.shape_factory import ShapeFactory
from pycatia.system_interfaces.any_object import AnyObject
from pycatia.system_interfaces.collection import Collection
from .axis_systems import AxisSystems
from .bodies import Bodies
from .body import Body
from .constraints import Constraints
from .factory import Factory
from .geometric_elements import GeometricElements
from .hybrid_bodies import HybridBodies
from .ordered_geometrical_sets import OrderedGeometricalSets
from .origin_ements import OriginElements


class Part(AnyObject):
    """
        .. note::
            CAA V5 Visual Basic help

                | The root level object inside a PartDocument object.Role: It aggregates
                | all the objects making up the part document.It provides many factories
                | and collections. The collections list only the direct children of the
                | part.activateLinkAnchor('Selection','Search','Selection.Search')allows
                | to get all objects of one type.

    """

    def __init__(self, com_part_object):
        super().__init__(com_part_object)
        self.part = com_part_object
        self.com_object = com_part_object

    @property
    def annotation_sets(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | AnnotationSets
                | o Property AnnotationSets() As Collection
                | 
                | Returns the collection object containing the annotation sets. All the
                | annotation sets that are aggregated in the part  might be accessed
                | thru that collection.  Example:The following example returns in
                | annotationSets the annotation sets of the partRoot part from the
                | partDoc part document:  Set partRoot = partDoc.Part Dim annotationSets
                | As AnnotationSets Set annotationSets = partRoot.AnnotationSets

        :return: Collection()
        """
        return Collection(self.part.AnnotationSets)

    @property
    def axis_systems(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | AxisSystems
                | o Property AxisSystems() As AxisSystems
                | 
                | Returns the collection object containing the coordinate systems. All
                | the coordinate systems that are aggregated in the part  might be
                | accessed thru that collection.  Example:The following example returns
                | in axisSystems the coordinate systems of the partRoot part from the
                | partDoc part document:  Set partRoot = partDoc.Part Dim axisSystems As
                | AxisSystems Set axisSystems = partRoot.AxisSystems

        :return: AxisSystems()
        """
        return AxisSystems(self.part.AxisSystems)

    @property
    def bodies(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Bodies
                | o Property Bodies() As Bodies
                | 
                | Returns the collection object containing the bodies that are direct
                | children of the part. It does not return all the bodies of the part,
                | particularly the bodies in a boolean operation.  Example:The following
                | example returns in bodiesColl the collection of the bodies of the
                | partRoot part from the partDoc part document:  Set partRoot =
                | partDoc.Part Set bodiesColl = partRoot.Bodies

        :return: Bodies()
        """
        return Bodies(self.part.Bodies)

    @property
    def constraints(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Constraints
                | o Property Constraints() As Constraints
                | 
                | Returns the collection object containing the part constraints. Only 3D
                | constraints are concerned here, 2D constraints are managed in
                | sketches.  Example:The following example returns in csts the
                | constraints of the partRoot part from the partDoc part document:  Set
                | partRoot = partDoc.Part Set csts = partRoot.Constraints

        :return: Constraints()
        """
        return Constraints(self.part.Constraints)

    @property
    def density(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Density
                | o Property Density() As double
                | 
                | Returns the part density.  Example:The following example displays the
                | density of the part:  Set partRoot = partDoc.Part MsgBox "The density
                | is " & partRoot.Density

        :return: float
        """
        return self.part.Density

    @property
    def file_name(self):
        """
        :return: str()
        """
        try:
            return self.part.ReferenceProduct.Parent.Name
        except AttributeError:
            return self.parent.Name

    @property
    def full_name(self):
        """
        :return: str()
        """
        try:
            return self.part.ReferenceProduct.Parent.FullName
        except AttributeError:
            return self.parent.FullName

    @property
    def geometric_elements(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | GeometricElements
                | o Property GeometricElements() As GeometricElements
                | 
                | Returns the collection object containing the part geometrical
                | elements. Only 3D elements are concerned here, 2D elements are managed
                | in sketches. The origin elements are also accessible thru that
                | collection.  Example:The following example returns in geomElts the 3D
                | elements of the partRoot part from the partDoc part document:  Set
                | partRoot = partDoc.Part Set geomElts = partRoot.GeometricElements

        :return: GeometricElements()
        """
        return GeometricElements(self.part.GeometricElements)

    @property
    def hybrid_bodies(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | HybridBodies
                | o Property HybridBodies() As HybridBodies
                | 
                | Returns the collection object containing the hybrid bodies that are
                | direct children of the part.  Example:The following example returns in
                | hybridBodiesColl the collection of hybrid bodies of the partRoot part
                | from the partDoc part document:  Set partRoot = partDoc.Part Set
                | hybridBodiesColl = partRoot.HybridBodies

        :return: HybridBodies()
        """
        return HybridBodies(self.part.HybridBodies)

    @property
    def hybrid_shape_factory(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | HybridShapeFactory
                | o Property HybridShapeFactory() As Factory
                | 
                | Returns the part hybrid shape factory. It allows the creation of
                | hybrid shapes in the part.
                |
                | Example:The following example returns in
                | hybridShapeFact the hybrid shape factory of the partRoot part from the
                | partDoc part document:
                | Set partRoot = partDoc.Part
                | Dim hybridShapeFact As Factory
                | Set hybridShapeFact = partRoot.HybridShapeFactory

        :return: HybridShapeFactory()
        """
        return HybridShapeFactory(self.part.HybridShapeFactory)

    @property
    def in_work_object(self):
        """
        .. note::
            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property InWorkObject() As AnyObject
                |
                |     Returns or sets the in work object of the part. The in work object is the
                |     object after which a new object is added.
                |
                |     Example:
                |
                |      Set partRoot = partDoc.Part
                |      Set partRoot.InWorkObject = cylindricPad
                |      If ( partRoot.InWorkObject <> cylindricPad ) Then
                |           MsgBox "There is a big problem"
                |      End If

        :return: AnyObject
        """

        return AnyObject(self.part.InWorkObject)

    @in_work_object.setter
    def in_work_object(self, value):
        """
        :param AnyObject value:
        """

        self.part.InWorkObject = value.com_object

    @property
    def main_body(self):
        """
        .. note::
            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property MainBody() As Body
                |
                |     Returns or sets the main body of the part.
                |
                |     Example:
                |         The following example returns the main body of the part of the current
                |         document.
                |
                |          Dim mainBody As Body
                |          Set mainBody=CATIA.ActiveDocument.Part.MainBody

        :return: Body
        """

        return Body(self.part.MainBody)

    @property
    def ordered_geometrical_sets(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | OrderedGeometricalSets
                | o Property OrderedGeometricalSets() As OrderedGeometricalSets
                | 
                | Returns the collection object containing the ordered geometrical sets
                | of the part.  Example:The following example returns in ogsColl the
                | collection of ordered geometrical sets of the partRoot part from the
                | partDoc part document:  Set partRoot = partDoc.Part Set ogsColl =
                | partRoot.OrderedGeometricalSets
        """
        return OrderedGeometricalSets(self.part.OrderedGeometricalSets)

    @property
    def origin_elements(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | OriginElements
                | o Property OriginElements() As OriginElements
                | 
                | Returns the object defining the part 3D reference axis system.
                |
                | Example:
                | The following example returns in originElts the origin of the
                | partRoot part from the partDoc part document:
                | Set partRoot = partDoc.Part
                | Set originElts = partRoot.OriginElements

        :return: OriginElements()
        """
        return OriginElements(self.part.OriginElements)

    @property
    def parameters(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Parameters
                | o Property Parameters() As Parameters
                |
                | Returns the collection object containing the part parameters. All the
                | parameters that are aggregated in the different objects of the part
                | might be accessed thru that collection.  Example:The following example
                | returns in params the parameters of the partRoot part from the partDoc
                | part document:  Set partRoot = partDoc.Part Dim params As Parameters
                | Set params = partRoot.Parameters

        :return: Parameters()
        """
        return Parameters(self.part.Parameters)

    @property
    def relations(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Relations
                | o Property Relations() As Relations
                |
                | Returns the collection object containing the part relations. All the
                | relations that are used to valuate  the parameters of the part might
                | be accessed thru that collection.  Example:The following example
                | returns in rels the relations of the partRoot part from the partDoc
                | part document:  Set partRoot = partDoc.Part Set rels =
                | partRoot.Relations

        :return: Relations()
        """
        return Relations(self.part.Relations)

    @property
    def shape_factory(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | ShapeFactory
                | o Property ShapeFactory() As Factory
                |
                | Returns the part shape factory. It allows the creation of shapes in
                | the part.  Example:The following example returns in shapeFact the
                | shape factory of the partRoot part from the partDoc part document:
                | Set partRoot = partDoc.Part Dim shapeFact As Factory   Set shapeFact =
                | partRoot.ShapeFactory

        :return: Factory()
        """
        return ShapeFactory(self.part.ShapeFactory)

    @property
    def sheet_metal_factory(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | SheetMetalFactory
                | o Property SheetMetalFactory() As Factory
                |
                | Returns the sheet metal factory of the part. It allows the creation of
                | sheet metal elements in the part.  Example:The following example
                | returns in sheetMetalFact the sheet metal factory of the partRoot part
                | from the partDoc part document:  Set partRoot = partDoc.Part Dim
                | sheetMetalFact As Factory Set sheetMetalFact =
                | partRoot.SheetMetalFactory

        """
        return Factory(self.part.SheetMetalFactory)

    @property
    def sheet_metal_parameters(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | SheetMetalParameters
                | o Property SheetMetalParameters() As AnyObject
                |
                | Returns the sheet metal parameters of the part.  Example:The following
                | example returns in sheetMetalParm the sheet metal parameters of the
                | partRoot part from the partDoc part document:  Set partRoot =
                | partDoc.Part Dim sheetMetalParm As SheetMetalParameters Set
                | sheetMetalFact = partRoot.SheetMetalParameters
        """
        return self.part.SheetMetalParameters

    @property
    def user_surfaces(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | UserSurfaces
                | o Property UserSurfaces() As Collection
                |
                | Returns the collection object containing the user surfaces. All the
                | user surfaces that are aggregated in the part  might be accessed thru
                | that collection.  Example:The following example returns in
                | userSurfaces the user surfaces of the partRoot part from the partDoc
                | part document:  Set partRoot = partDoc.Part Dim userSurfaces As
                | UserSurfaces Set userSurfaces = partRoot.UserSurfaces

        """
        return self.part.UserSurfaces

    def activate(self, i_object):
        """
        .. note::
            CAA V5 Visual Basic help

                | Activate
                | o Sub Activate(    AnyObject    iObject)
                |
                | Unsuppresses an object for the update process. A unsuppressed object
                | is again taken into account  for the calculation of the part.
                |
                | Parameters:
                | iObject
                |     The object to unsuppress for the update process
                | Examples:
                | The following example unsuppresses the pad1 pad:
                | Set partRoot = partDoc.Part
                | Set pad1 = partRoot.FindObjectByName("Pad.1")
                | partRoot.Activate(pad1)

        """
        self.part.Activate(i_object.com_object)

    def create_reference_from_b_rep_name(self, i_label, i_object_context):
        """
        .. note::
            CAA V5 Visual Basic help

                | CreateReferenceFromBRepName
                | o Func CreateReferenceFromBRepName(    CATBSTR    iLabel,
                |                                        AnyObject    iObjectContext) As Reference
                |
                | Creates a reference from a GenericNaming label. This allows
                | manipulation of B-Rep (Type Functinal and Relimited) that are not easy
                | to access.
                |
                | Parameters:
                | iLabel
                |    The GenericNaming identification for an object.
                |    This is a cryptic form for "the edge surrounded by the face extruded
                |    from line.12 of sketch.4 and the face...")
                |
                |  iObjectContext
                |    The Object Context of Resolution
                |    This is the feature used for label GenericNaming resolution
                |  Returns:
                |   The reference to a B-Rep sub-element such a face or an edge

        :param str i_label:
        :param AnyObject() i_object_context:
        :return: Reference()
        """
        return Reference(self.part.CreateReferenceFromBRepName(i_label, i_object_context.com_object))

    def create_reference_from_name(self, i_label):
        """
        .. note::
            CAA V5 Visual Basic help

                | CreateReferenceFromName
                | o Func CreateReferenceFromName(    CATBSTR    iLabel) As Reference
                |
                | Creates a reference from a GenericNaming label. This allows
                | manipulation of B-Rep (type Functional Only) that are not easy to
                | access.
                |
                | Parameters:
                | iLabel
                |    The GenericNaming identification for an object.
                |    This is a cryptic form for "the edge surrounded by the face extruded
                |    from line.12 of sketch.4 and the face...")
                |  Returns:
                |   The reference to a B-Rep sub-element such a face or an edge

        :param str i_label:
        :return: Reference()
        """
        return Reference(self.part.CreateReferenceFromName(i_label))

    def create_reference_from_object(self, i_object):
        """
        .. note::
            CAA V5 Visual Basic help

                | CreateReferenceFromObject
                | o Func CreateReferenceFromObject(    AnyObject    iObject) As Reference
                |
                | Creates a reference from a operator. Use of reference allows a uniform
                | handling of B-Rep and non B-Rep objects.
                |
                | Parameters:
                | iObject
                |    The geometric object to be referenced.
                |    It can be a plane, a line or a point.
                |
                |  Returns:
                |   The reference to the object. This way, a direction can be either an edge of a pad or a 3D line.

        :param AnyObject() i_object:
        :return: Reference()
        """
        return Reference(self.part.CreateReferenceFromObject(i_object.com_object))

    def deactivate(self, i_object):
        self.inactivate(i_object)

    def find_object_by_name(self, i_obj_name):
        """
        .. note::
            CAA V5 Visual Basic help

                | FindObjectByName
                | o Func FindObjectByName(    CATBSTR    iObjName) As AnyObject
                |
                | Finds an object that is not a collection by its name. Scan in depth
                | among all the direct and indirect children (expensive, but hard to
                | escape).
                |
                | Parameters:
                | iObjName
                |    The name to be searched
                |
                | Returns:
                |   The object, if found
                |
                | Examples:
                | The following example tests if the object was found:
                | Set partRoot = partDoc.Part
                | Set obj = partRoot.FindObjectByName("Wrong name")
                | If TypeName(obj)="Nothing" Then
                | MsgBox "Object not found"
                | End If

        :param str i_obj_name:
        :return: AnyObject()
        """
        if self.part.FindObjectByName(i_obj_name):
            return AnyObject(self.part.FindObjectByName(i_obj_name))

        return None

    def get_customer_factory(self, i_factory_iid):
        """
        .. note::
            CAA V5 Visual Basic help

                | GetCustomerFactory
                | o Func GetCustomerFactory(    CATBSTR    iFactoryIID) As Factory
                |
                | Returns a customer factory from a code string defined by the customer.
                | It allows a customer to define its own factory to create its own
                | objects.


                | Parameters:
                | iFactoryIID
                | 	The code name of the factory

        :param str i_factory_iid:
        :return: Factory()
        """
        return Factory(self.part.GetCustomerFactory(i_factory_iid))

    def inactivate(self, i_object):
        """
        .. note::
            CAA V5 Visual Basic help

                | Inactivate
                | o Sub Inactivate(    AnyObject    iObject)
                |
                | Suppresses an object from being updated. A suppressed object is not
                | taken into account  for the calculation of the part.
                |
                | Parameters:
                | iObject
                |    The object to suppress from being updated
                |
                | Examples:
                | The following example suppresses the pad1 pad
                | from being updated:
                | Set partRoot = partDoc.Part
                | Set pad1 = partRoot.FindObjectByName("Pad.1")
                | partRoot.Inactivate(pad1)

        :param AnyObject() i_object:
        """
        self.part.Inactivate(i_object.com_object)

    def is_inactive(self, i_object):
        """
        .. note::
            CAA V5 Visual Basic help

                | IsInactive
                | o Func IsInactive(    AnyObject    iObject) As boolean
                |
                | Indicates whether an object is deactivated. A deactivated object is
                | not taken into account for the calculation of the part.


                | Parameters:
                | iObject
                |    The object to examine
                |
                | Examples:
                | The following example returns in isInactive
                | whether the pad1 pad is deactivated:
                | Set partRoot = partDoc.Part
                | Set pad1 = partRoot.FindObjectByName("Pad.1")
                | isInactive = partRoot.IsInactive(pad1)

        :param AnyObject() i_object:
        :return: bool
        """
        return self.part.IsInactive(i_object.com_object)

    def is_up_to_date(self, i_object):
        """
        .. note::
            CAA V5 Visual Basic help

                | IsUpToDate
                | o Func IsUpToDate(    AnyObject    iObject) As boolean
                |
                | Indicates whether an object needs to be updated. An object which is
                | not up-to-date has not be calculated  with the last specifications.
                |
                | Parameters:
                | iObject
                |    The object to examine
                |
                | Examples:
                | The following example returns in isuptodate whether
                | the pad1 pad is up-to-date:
                | Set partRoot = partDoc.Part
                | Set pad1 = partRoot.FindObjectByName("Pad.1")
                | isuptodate = partRoot.IsUpToDate(pad1)

        :param AnyObject() i_object:
        :return: bool
        """
        return self.part.IsUpToDate(i_object.com_object)

    def path(self):
        """

        Returns the pathlib.Path() object of the document fullname.
        example e://users//psr//Parts//MyNicePart.CATPart
        >>> Part.path().name
        MyNicePart.CATPart
        >>> Part.path().parent
        e://users//psr//Parts//
        >>> Part.path().suffix
        .CATPart

        :return: Path()
        """
        return Path(self.full_name)

    def update(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Update
                | o Sub Update()
                | 
                | Updates of the part result with respect to its specifications. Any
                | composing specification that hasn't its result up-to-date will
                | recompute it, thus propagating changes to the whole part.  Example:The
                | following example update the part:  Set partRoot = partDoc.Part
                | partRoot.Update

        """
        self.part.Update()

    def update_object(self, i_object):
        """
        .. note::
            CAA V5 Visual Basic help

                | UpdateObject
                | o Sub UpdateObject(    AnyObject    iObject)
                | 
                | Updates an object with respect to its specifications. Any composing
                | specification of the object that hasn't  its result up-to-date will
                | recompute it, thus propagating changes to the object.
                |
                | Parameters:
                | iObject
                |    The object to be updated
                |
                | Examples:
                | The following example updates Pad.1:
                | Set partRoot = partDoc.Part
                | Set pad1 = partRoot.FindObjectByName("Pad.1")
                | partRoot.UpdateObject(pad1)

        :param AnyObject() i_object:
        """
        return self.part.UpdateObject(i_object.com_object)

    def __repr__(self):
        return f'Part(name="{self.name}")'



#! /usr/bin/python3.6
# module initially auto generated using V5Automation.chm from CATIA V5 R25

from pycatia.system_interfaces.any_object import AnyObject


class HybridShape(AnyObject):
    """
        .. note::
            CAA V5 Visual Basic help

                | Represents the hybrid shape object.

    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.hybridshape = com_object

    @property
    def thickness(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Thickness
                | o Property Thickness() As HybridShape
                | 
                | Returns the thickness of the hybrid shape.   The thickness is a
                | CATIAHybridShapeThickness.  Example:The following example returns the
                | thickness ExtrudeThickness of the extrude Extrude.1 as the origin
                | point of the axis system AxisSystem0:  Dim Extrude1 As AnyObject Set
                | Extrude1 = HybridBody1.HybridShapes.Item  ( "Extrude.1" )  Dim
                | Thickness1 As HybridShapeThickness Set Thickness1 = Extrude1.Thickness

        """
        return HybridShape(self.hybridshape.Thickness)

    def append_hybrid_shape(self, i_hybrid_shape):
        """
        .. note::
            CAA V5 Visual Basic help

                | AppendHybridShape
                | o Sub AppendHybridShape(    HybridShape    iHybridShape)
                | 
                | Appends a hybrid shape to another hybrid shape.
                |
                | Parameters:
                | iHybridShape
                |    The hybrid shape to append.
                |
                | Examples:
                |
                | This example appends the hybrid shape newHybridShape
                | to the hybrid shape oldHybridShape:
                | 
                | oldHybridShape.AppendHybridShape (newHybridShape)

        """
        return self.hybridshape.AppendHybridShape(i_hybrid_shape.com_object)

    def compute(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Compute
                | o Sub Compute()
                | 
                | Computes the result of the hybrid shape.


                | Parameters:


        """
        return self.hybridshape.Compute()

    def __repr__(self):
        return f'HybridShape(name="{self.name}")'



#! /usr/bin/python3.6
# module initially auto generated using V5Automation.chm from CATIA V5 R25

from pycatia.system_interfaces.collection import Collection
from .hybrid_body import HybridBody


class HybridBodies(Collection):
    """
        .. note::
            CAA V5 Visual Basic help

                | A collection of the HybridBody objects.

    """

    def __init__(self, collection_com_object):
        super().__init__(collection_com_object, child_object=HybridBody)
        self.hybrid_bodies = collection_com_object

    def add(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Add
                | o Func Add() As HybridBody
                | 
                | Creates a new hybrid body and adds it to the HybridBodies collection.
                | This body becomes the current one
                | Returns:  The created body
                |
                | Example: The following example creates a body named newHybridBody in
                | the hybrid body collection of the rootPart part in the partDoc part
                | document. NewPartBody becomes the current body in partDoc.
                |
                | Set NewPartBody = rootPart.Bodies.AddPartBody()


        :return: HybridBody()
        """
        return self.child_object(self.hybrid_bodies.Add())

    def item(self, i_index):
        """
        .. note::
            CAA V5 Visual Basic help

                | Item
                | o Func Item(    CATVariant    iIndex) As HybridBody
                | 
                | Returns a body using its index or its name from the Bodies collection.
                |
                | Parameters:
                | iIndex
                |    The index or the name of the hybrid body to retrieve from
                |    the collection of hybrid bodies.
                |    As a numeric, this index is the rank of the hybrid body
                |    in the collection.
                |    The index of the first hybrid body in the collection is 1, and
                |    the index of the last hybrid body is Count.
                |    As a string, it is the name you assigned to the hybrid body using
                |    the 
                | 
                |  activateLinkAnchor('AnyObject','Name','AnyObject.Name')  property. 
                |    Returns:
                |   The retrieved hybrid body
                |
                | Examples:
                | 
                | 
                | This example retrieves in ThisHybridBody the fifth hybrid body
                | in the collection and in ThatHybridBody the hybrid body
                | named MyHybridBody in the hybrid body collection of the partDoc
                | part document.
                | 
                | Set hybridBodyColl = partDoc.Part.HybridBodies
                | Set ThisHybridBody = hybridBodyColl.Item(5)
                | Set ThatHybridBody = hybridBodyColl.Item("MyHybridBody")


        """
        return HybridBody(self.hybrid_bodies.Item(i_index))

    def __repr__(self):
        return f'HybridBodies(name="{self.name}")'



#! /usr/bin/python3.6
# module initially auto generated using V5Automation.chm from CATIA R25 on 2020-05-22 12:54:36.956119

from .edge import Edge


class TriDimFeatEdge(Edge):
    """
        .. note::
            CAA V5 Visual Basic help

                | 1-D boundary belonging to a feature whose topological result is three
                | dimensional.Role:
                | ThisactivateLinkAnchor('Boundary','','Boundary')object may be, for
                | example, the edge of a Pad.  You will create a TriDimFeatEdge object
                | using theactivateLinkAnchor('Shapes','GetBoundary','Shapes.GetBoundary
                | '),activateLinkAnchor('HybridShapes','GetBoundary','HybridShapes.GetBo
                | undary'),activateLinkAnchor('Sketches','GetBoundary','Sketches.GetBoun
                | dary')oractivateLinkAnchor('Selection','SelectElement2','Selection.Sel
                | ectElement2')method. Then, you pass it to the operator (such asactivat
                | eLinkAnchor('ShapeFactory','AddNewEdgeFilletWithConstantRadius','Shape
                | Factory.AddNewEdgeFilletWithConstantRadius')).  The lifetime of a
                | TriDimFeatEdge object is limited,
                | seeactivateLinkAnchor('Boundary','','Boundary').

    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.tri_dim_feat_edge = com_object

    def __repr__(self):
        return f'TriDimFeatEdge(name="{self.name}")'



#! /usr/bin/python3.6
# module initially auto generated using V5Automation.chm from CATIA R25

from pycatia.system_interfaces.collection import Collection
from .ordered_geometrical_set import OrderedGeometricalSet


class OrderedGeometricalSets(Collection):
    """
        .. note::
            CAA V5 Visual Basic help

                | A collection of the OrderedGeometricalSet objects.

    """

    def __init__(self, collection_com_object):
        super().__init__(collection_com_object, child_object=OrderedGeometricalSet)
        self.ordered_geometrical_sets = collection_com_object

    def add(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Add
                | o Func Add() As
                | 
                | Creates a new ordered geometrical set and adds it to the
                | OrderedGeometricalSets collection. This ordered geometrical set becomes
                | the current one  Returns:  The created ordered geometrical set
                |
                | Example:
                | The following example creates a ordered geometrical set named
                | newOrderedGeometricalSet in the ordered geometrical set collection of
                | the rootPart part in the partDoc part document. NewPartBody becomes
                | the in work object of partDoc.
                | Set NewPartBody = rootPart.OrderedGeometricalSets.Add()

        """
        return self.child_object(self.ordered_geometrical_sets.Add())

    def item(self, i_index):
        """
        .. note::
            CAA V5 Visual Basic help

                | Item
                | o Func Item(iIndex) As OrderedGeometricalSet
                | 
                | Returns a ordered geometrical set using its index or its name from the
                | ordered geometrical set collection.


                | Parameters:
                | iIndex
                |    The index or the name of the ordered geometrical set to retrieve from
                |    the collection of ordered geometrical sets.
                |    As a numeric, this index is the rank of the ordered geometrical set
                |    in the collection.
                |    The index of the first ordered geometrical set in the collection is 1, and
                |    the index of the last ordered geometrical set is Count.
                |    As a string, it is the name you assigned to the ordered geometrical set using
                |    the 
                | 
                |  property. 
                |    Returns:
                |   The retrieved ordered geometrical set


                | Examples:
                | 
                | 
                | This example retrieves in ThisOrderedGeometricalSet the fifth ordered geometrical set
                | in the collection and in ThatOrderedGeometricalSet the ordered geometrical set
                | named MyOrderedGeometricalSet in the ordered geometrical set collection of the partDoc
                | part document.
                | 
                | Set orderedGeometricalSetColl = partDoc.Part.OrderedGeometricalSets
                | Set ThisOrderedGeometricalSet = orderedGeometricalSetColl.Item(5)
                | Set ThatOrderedGeometricalSet = orderedGeometricalSetColl.Item("MyOrderedGeometricalSet")

        :return: OrderedGeometricalSet()
        """
        return self.child_object(self.ordered_geometrical_sets.Item(i_index))

    def __repr__(self):
        return f'OrderedGeometricalSets(name="{self.name}")'



#! /usr/bin/python3.6

from pathlib import Path

from pycatia.system_interfaces.any_object import AnyObject
from pycatia.product_structure_interfaces.analyze import Analyze
from pycatia.base_interfaces.move import Move
from pycatia.in_interfaces.position import Position
from pycatia.enumeration.enumeration_types import cat_file_type
from pycatia.enumeration.enumeration_types import cat_work_mode_type
from pycatia.enumeration.enumeration_types import cat_rep_type


class Product(AnyObject):
    def __init__(self, com_object):
        """

        .. note::
            CAA V5 Visual Basic help

            Represents the product.

            The product is the object that helps you model your real products by building a
            tree structure whose nodes are product objects. Each of them may contain other
            product objects gathered in a product collection. The terminal product objects in
            the tree structure have no aggregated product collection. Even if all products are
            located somewhere in the product tree structure, some of them can be used as reference
            products to create other products named components, which are instances of the
            reference product. For example, the left front wheel in a car can be used as reference
            to create the other wheels. Be careful: some properties and methods are dedicated to
            reference objects only, and some others are for components only. This is clearly stated
            for each property or method concerned.

        :param com_object: Product COM object
        """

        super().__init__(com_object)
        self.product = com_object

    @property
    def analyze(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Analyze
                | o Property Analyze() As Analyze
                |
                | Returns the Analyze object associated to the current product.
                | Example:    This example retrieves in EngineAnalysis the Analyze
                | object of the Engine product.  Dim EngineAnalysis As Analyze Set
                | EngineAnalysis = Engine.Analyze


        :return: analyze object
        """

        return Analyze(self.product.Analyze)

    @property
    def definition(self):
        """

        .. note::
            CAA V5 Visual Basic help
            Property Definition( ) As CATBSTR

            Returns or sets the product's definition.
            Definition is valid for reference products only.
            | Example:
            | This example retrieves the definition of the Engine product in EngineDef.
            |     EngineDef = Engine.Definition


        :return: str()
        """

        return self.product.Definition

    @definition.setter
    def definition(self, definition):
        """
        :param str() definition:
        """

        self.product.Definition = definition

    @property
    def description_instance(self):
        """

        .. note::
            CAA V5 Visual Basic help

            Property DescriptionInst( ) As CATBSTR

            Returns or sets the product's description for a component product.
            DescriptionInst is valid for component products only.
            The description is a comment assigned to the component product to help describe or qualify it.
            | Example:
            | This example sets the description for the EngineComp product.
            |     Desc = "This is the Engine component product description"
            |     EngineComp.DescriptionInst(Desc)

        :return: str()
        """

        return self.product.DescriptionInst

    @description_instance.setter
    def description_instance(self, description_instance):
        """
        :param str() description_instance:
        """

        self.product.DescriptionInst = description_instance

    @property
    def description_reference(self):
        """

        .. note::
            CAA V5 Visual Basic help

            Property DescriptionRef( ) As CATBSTR

            Returns or sets the product's description for a reference product.
            DescriptionRef is valid for reference products only.
            The description is a comment assigned to the reference product to help describe or qualify it.
            | Example:
            | This example sets the description for the Engine product.
            |      Desc = "This is the Engine reference product description"
            |      Engine.DescriptionRef(Desc)


        :return: str()
        """

        return self.product.DescriptionRef

    @description_reference.setter
    def description_reference(self, description_reference):
        """
        :param str description_reference:
        """
        self.product.DescriptionRef = description_reference

    @property
    def move(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Move
                | o Property Move() As Move
                |
                | Returns the product's move object. The move object is aggregated by
                | the product object and itself aggregates a movable object to which you
                | can apply a move transformation by means of an isometry matrix. It
                | moves your product master shape representation according to this
                | isometry.  Example:    This example retrieves the move object for the
                | Engine product.  Dim EngineMoveObject As Move Set EngineMoveObject =
                | Engine.Move

        """
        return Move(self.product)

    @property
    def file_name(self):
        """

        :return: str()
        """

        return self.product.ReferenceProduct.Parent.Name

    @property
    def full_name(self):
        """

        :return: str()
        """

        return self.product.ReferenceProduct.Parent.FullName

    def path(self):
        """
       
        Returns the pathlib.Path() object of the document fullname.

        example e://users//psr//Parts//MyNiceProduct.CATProduct
        >>> Product.path().name
        >>> # MyNiceProduct.CATProduct
        >>> Product.path().parent
        >>> # e://users//psr//Parts//
        >>> Product.path().suffix
        >>> # .CATProduct

        :return: Path()
        """

        return Path(self.full_name)

    @property
    def nomenclature(self):
        """

        .. note::
            CAA V5 Visual Basic help

            Property Nomenclature( ) As CATBSTR

            Returns or sets the product's nomenclature.
            Nomenclature is valid for reference products only.
            According to the STEP AP203, the nomenclature is
            "a name by which the part is commonly known within an organization".
            | Example:
            | This example retrieves the nomenclature the Engine product in EngineNom.
            | EngineNom = Engine.Nomenclature

        :return: str()
        """

        return self.product.nomenclature

    @nomenclature.setter
    def nomenclature(self, nomenclature):
        """
        :param str() nomenclature:
        """

        self.product.nomenclature = nomenclature

    @property
    def parameters(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Parameters
                | o Property Parameters() As Parameters
                |
                | Returns the collection object containing the product parameters. All
                | the parameters that are aggregated in the different objects of the
                | product might be accessed through that collection.  Example:The
                | following example returns in params the parameters of the productRoot
                | product from the productDoc product document:  Set productRoot =
                | productDoc.Product Set params = productRoot.Parameters

        """
        # todo: not implemented yet.
        return self.product.Parameters

    @property
    def part_number(self):
        """

        .. note::
            CAA V5 Visual Basic help

            Property PartNumber( ) As CATBSTR

            Returns or sets the product's part number.
            PartNumber is valid for reference products only.
            | Example:
            | This example sets the Engine product's part number to A120-253X-7.
            |     Engine.PartNumber("A120-253X-7")


        :return: str()
        """

        return self.product.PartNumber

    @part_number.setter
    def part_number(self, part_number):
        """
        :param str() part_number:
        """

        self.product.PartNumber = part_number

    @property
    def position(self):
        """
         .. note::
             CAA V5 Visual Basic help

                 | Position
                 | o Property Position() As Position
                 |
                 | Returns the product's position object. The position object is the
                 | object aggregated by the product object that holds the position of the
                 | master shape representation in the space.  Example:    This example
                 | retrieves the position object for the Engine product.  Dim
                 | EnginePositionObject As Position Set EnginePositionObject =
                 | Engine.Position

         """

        return Position(self.product)

    @property
    def products(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Products
                | o Property Products() As Products
                |
                | Returns the collection of products contained in the current product.
                | Example:    This example retrieves in EngineChildren the collection of
                | products contained in the Engine product.  Dim EngineChildren As
                | Products Set EngineChildren = Engine.Products

        """
        return self.product.Products

    @property
    def publications(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Publications
                | o Property Publications() As Publications
                |
                | Returns the collection of publications managed by the product.

        """
        return self.product.Publications

    @property
    def reference_product(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | ReferenceProduct
                | o Property ReferenceProduct() As Product
                |
                | Returns the Reference Product of this instance.

        """
        return self.product.ReferenceProduct

    @property
    def relations(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Relations
                | o Property Relations() As Relations
                |
                | Returns the collection object containing the product relations. All
                | the relations that are used to valuate the parameters of the product
                | might be accessed thru that collection.  Example:The following example
                | returns in rels the relations of the productRoot product from the
                | productDoc product document:  Set productRoot = productDoc.Product Set
                | rels = productRoot.Relations

        """
        return self.product.Relations

    @property
    def revision(self):
        """

        .. note::
            CAA V5 Visual Basic help

            Property Revision( ) As CATBSTR

            Returns or sets the product's revision number.
            Revision is valid for reference products only.
            | Example:
            | This example sets the Engine product's revision number to 3A.
            |     Engine.Revision("3A")

        :return: str
        """

        return self.product.Revision

    @revision.setter
    def revision(self, revision):
        """
        :param str() revision:
        """

        self.product.Revision = revision

    @property
    def source(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Source
                | o Property Source() As CatProductSource
                |
                | Returns or sets the product's source. Source is valid for reference
                | products only.  According to the STEP AP203, the source is the "design
                | organization's plan for obtaining the product". The source can take
                | the values catProductMade if the product is made internally,
                | catProductBought if it is purchased from a vendor, or
                | catProductUnknown if its origin is not determined.  Example:    This
                | example sets the source for the Engine product to catProductMade.
                | Engine.Source(catProductMade)

        """
        return self.product.Source

    @property
    def user_ref_properties(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | UserRefProperties
                | o Property UserRefProperties() As Parameters
                |
                | Returns the collection object containing the product properties. All
                | the user defined properties that are created in the reference product
                | might be accessed through that collection.  Only available on
                | reference products.  Example:The following example returns in
                | UserProps the properties of the productRoot product from the
                | productDoc product document:  Set productRoot = productDoc.Product Set
                | UserProps = productRoot.UserRefProperties

        """
        return self.product.UserRefProperties

    def attributes(self):
        """
        Returns a string describing the products attributes.

        :return: str
        """

        return (
            "(Product) Attributes... \n"
            f"File Name:             {self.file_name}\n"
            f"Name:                  {self.name}\n"
            f"Part Number:           {self.part_number}\n"
            f"Revision:              {self.revision}\n"
            f"Definition:            {self.definition}\n"
            f"Nomenclature:          {self.nomenclature}\n"
            f"Description Instance:  {self.description_instance}\n"
            f"Description Reference: {self.description_reference}\n"
            f"Reference:             {self.reference_product}\n"
            f"Is CATProduct:         {self.is_catproduct()}\n"
            f"Is CATPart:            {self.is_catpart()}"
        )

    def activate_default_shape(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | ActivateDefaultShape
                | o Sub ActivateDefaultShape()
                |
                | Activate default shape.

        """
        return self.product.ActivateDefaultShape()

    def activate_shape(self, shape_name):
        """
        .. note::
            CAA V5 Visual Basic help

                | ActivateShape
                | o Sub ActivateShape(    CATBSTR    ShapeName)
                |
                | Activate one shape.
                |
                | Parameters:
                | ShapeName
                |            The name of the shape.


        """
        return self.product.ActivateShape(shape_name)

    @staticmethod
    def activate_terminal_node(products):
        """
        Method to 'Activate Terminal Node'.

        Loops through ALL products in product and activates_default_shape().

        :param products: self.get_products()
        """

        # noinspection PyShadowingNames
        def loop_d_loop(products):

            for product in products:
                if product.is_catpart():
                    product.activate_default_shape()
                elif product.is_catproduct():
                    loop_d_loop(product.get_products())

        loop_d_loop(products)

    def add_master_shape_representation(self, i_shape_path_name):
        """
        .. note::
            CAA V5 Visual Basic help

                | AddMasterShapeRepresentation
                | o Sub AddMasterShapeRepresentation(    CATBSTR    iShapePathName)
                |
                | Adds the master shape representation to the product. The master shape
                | representation is the object that gives a geometric shape and allows
                | the visualization of the product. It can be a CATIA V4 model, a VRML
                | file, or any other type of document that can be displayed. In a multi
                | representation  context, the master shape representation is the most
                | meaningful representation of the product according to the user. This
                | is the default shape for the multi representation. Note: This master
                | shape representation is optional.


                | Parameters:
                | iShapePathName
                |    The path name where the master shape representation can be found
                |
                | Examples:
                |
                | This example adds the e:/Models/Engine.model as
                | the master shape representation to the Engine product.
                |
                | Engine.AddMasterShapeRepresentation("e:/Models/Engine.model")

        """
        # todo: not yet implemented
        return self.product.AddMasterShapeRepresentation(i_shape_path_name)

    def add_shape_representation(
            self, i_shape_path_name, i_shape_name, i_rep_behavior, i_context
    ):
        """
        .. note::
            CAA V5 Visual Basic help

                | AddShapeRepresentation
                | o Sub AddShapeRepresentation(    CATBSTR    iShapePathName,
                |                                  CATBSTR    iShapeName,
                |                                  CatRepType    iRepBehavior,
                |                                  boolean    iContext)
                |
                | Adds a representation to the product with a specific behavior. A
                | representation is the object that gives a geometric shape and allows
                | the visualization of the product. It can be  a CATIA V4 model, a VRML
                | file, or any other type of document that can be displayed. Note: The
                | possible behavior supported are : 3D, 2D and text. The representation
                | can also be added within a context or not. A representation on a
                | product is optional, but many representation with  different behavior
                | (or the same) is supported
                |
                | Parameters:
                | iShapePathName
                |    The path name where the representation can be found
                |  iShapeName
                |    The name that is given to the representation
                |    This name is a user free choice
                |  iRepBehavior
                |    The behavior of the added representation.
                |    It can take the values catRep3D if the representation is a 3D one,
                |    catRep2D if the representation is a 2D one,
                |    or catRepText if the representation is a text one.
                |  iContext
                |    A condition to specify if the added representation can be
                |    displayed with the representation of other products.
                |
                | Examples:
                | This example adds the e:/Models/Engine.model as
                | a 3D representation to the Engine product within an assembly context.
                | Engine.AddShapeRepresentation("e:/Models/Engine.model","MyShape",catRep3D,TRUE)

        """
        # todo: not yet implemented
        return self.product.AddShapeRepresentation(
            i_shape_path_name, i_shape_name, i_rep_behavior, i_context
        )

    def apply_work_mode(self, new_mode):
        """
        Applies the work mode defined by new_mode.

        new_mode must be a string that matches KEY value of cat_work_types

        .. note::
            CAA V5 Visual Basic help

                | ApplyWorkMode
                | o Sub ApplyWorkMode(    CatWorkModeType    newMode)
                |
                | Applies a new working mode.
                |
                | Parameters:
                | newMode
                |            The new working mode.

        :param str new_mode:
        """
        return self.product.ApplyWorkMode(cat_work_mode_type.index(new_mode))

    def connections(self, i_connections_type):
        """
        .. note::
            CAA V5 Visual Basic help

                | Connections
                | o Func Connections(    CATBSTR    iConnectionsType) As Collection
                |
                | Returns the product's constraints. The constraint collection of a
                | product gathers the constraints this product should respect to be
                | positioned in the space.  Example:    This example retrieves the
                | constraint collection for the Engine product.  Dim EngineConstraints
                | As Collection Set EngineConstraints = Engine.Constraints

        """
        return self.product.Connections(i_connections_type)

    def create_reference_from_name(self, i_label):
        """
        .. note::
            CAA V5 Visual Basic help

                | CreateReferenceFromName
                | o Func CreateReferenceFromName(    CATBSTR    iLabel) As Reference
                |
                | Creates a reference from a name. A reference is an object that can
                | stand for any geometrical object. Creating references is necessary for
                | adding constraints between two components using Brep elements of the
                | representations of these components.
                |
                | Parameters:
                | iLabel
                |    The path of the Brep element to use in the constraint.
                |    This path is passed as a character string comprising the component
                |    path from the root product to the component concerned, concatenated
                |    to the Brep element path in the product's representation.
                |    Components are separated using "/", and the product path is separated
                |    from the Brep using "/!". For separating parameter from product path use "/".
                |
                |  Returns:
                |   The created reference
                |
                | Examples:
                | This example creates a reference from the path of a Brep element
                | in the Prod2 product located below the Root root
                | product. The face is located in the Pad.1 pad and limited by the
                | Circle.1 circle.
                |
                | Dim Ref As Reference
                | Ref = Prod2.CreateReferenceFromName("Root/Prod2/!Face:(Brp:(Pad.1:0(Brp:(Circle.1))):None())")

        """
        return self.product.CreateReferenceFromName(i_label)

    def deactivate_default_shape(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | DesactivateDefaultShape
                | o Sub DesactivateDefaultShape()
                |
                | Deactivate default shape.

        """
        return self.product.DesactivateDefaultShape()

    def deactivate_shape(self, shape_name):
        """
        .. note::
            CAA V5 Visual Basic help

                | DesactivateShape
                | o Sub DesactivateShape(    CATBSTR    ShapeName)
                |
                | Deactivate one shape.
                |
                | Parameters:
                | ShapeName
                |            The name of the shape.

        """
        return self.product.DesactivateShape(shape_name)

    def extract_bom(self, i_file_type, i_file):

        """

        i_file_type must be a string that matches KEY value of cat_file_types

        .. note::
            CAA V5 Visual Basic help

                | ExtractBOM
                | o Sub ExtractBOM(    CatFileType    iFileType,
                |                      CATBSTR    iFile)
                |
                | Extracts the product's contents as a bill of materials (BOM). The bill
                | of material displays, for every sub-assembly in the product, the one
                | level depth components and some of their properties.
                |
                | Parameters:
                | iFileType
                |    Set this parameter to catFileTypeHTML to save to the html format.
                |
                |    Set this parameter to catFileTypeTXT to save to the text format.
                |
                |    The catFileTypeMotif should not be used.
                |
                |  iFile
                |    File where the bill of material will be saved


        """
        return self.product.ExtractBOM(cat_file_type[i_file_type], i_file)

    def get_active_shape_name(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | GetActiveShapeName
                | o Func GetActiveShapeName() As CATBSTR
                |
                | Returns the name of the active shape.  Returns:  oShapeName
                | The name of the active shape.

        """
        return self.product.GetActiveShapeName()

    def get_all_shapes_names(self, olistshape):
        """
        .. note::
            CAA V5 Visual Basic help

                | GetAllShapesNames
                | o Sub GetAllShapesNames(    CATSafeArrayVariant    olistshape)
                |
                | List the name of all shapes.  Returns:  olistshape           The list
                | of the names  The tab olistshape has to be allocated with a size given
                | by GetNumberOfShapes.

        """
        return self.product.GetAllShapesNames(olistshape)

    def count_children(self):
        """

       :return: int()
       """

        return self.product.Products.Count

    def get_child(self, index):
        """
        :return: Product()
        """
        return Product(self.product.Products.Item(index + 1))

    def get_children(self):
        """

        :return: list(Product())
        """

        children = list()

        if self.has_children():
            for i in range(self.product.Products.Count):
                child = Product(self.product.Products.Item(i + 1))
                children.append(child)

        return children

    def get_default_shape_name(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | GetDefaultShapeName
                | o Func GetDefaultShapeName() As CATBSTR
                |
                | Returns the default shape.  Returns:  oShapeName           The name of
                | the default shape.

        """
        return self.product.GetDefaultShapeName()

    def get_master_shape_representation(self, i_load_if_necessary):
        """
        .. note::
            CAA V5 Visual Basic help

                | GetMasterShapeRepresentation
                | o Func GetMasterShapeRepresentation(    boolean    iLoadIfNecessary) As CATBaseDispatch
                |
                | Retrieves the product's master shape representation.
                |
                | Parameters:
                | iLoadIfNecessary
                |    Parameter to set to True if the master shape representation
                |    should be loaded to determine if it exists, or to False otherwise.
                |
                | Examples:
                |
                | This example retrieves in MSRep the
                | Engine product's master shape representation.
                |
                | Dim MSRep As Object
                | Set MSRep = Engine.GetMasterShapeRepresentation(True)


        """
        return self.product.GetMasterShapeRepresentation(i_load_if_necessary)

    def get_master_shape_representation_path_name(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | GetMasterShapeRepresentationPathName
                | o Func GetMasterShapeRepresentationPathName() As CATBSTR
                |
                | Retrieves the product's master shape representation pathname.
                | Example:    This example retrieves in MSRep the  Engine product's
                | master shape representation.  Set MSRepPath =
                | Engine.GetMasterShapeRepresentationPathName

        """
        return self.product.GetMasterShapeRepresentationPathName()

    def get_number_of_shapes(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | GetNumberOfShapes
                | o Func GetNumberOfShapes() As short
                |
                | Returns the number of Shapes  Returns:  oNbShapes           The number
                | of Shapes.

        """
        return self.product.GetNumberOfShapes()

    def get_products(self):
        """
        Returns a list of Product().

        :return: [Product()]
        """
        products = []

        for i in range(0, self.product.Products.Count):
            product = Product(self.product.Products.Item(i + 1))
            products.append(product)

        return products

    def get_shape_path_name(self, i_shape_name):
        """
        .. note::
            CAA V5 Visual Basic help

                | GetShapePathName
                | o Func GetShapePathName(    CATBSTR    iShapeName) As CATBSTR
                |
                | Returns the path name of a shape for a given shape name.
                |
                | Parameters:
                | iShapeName
                |    The name of the shape.
                |
                |  Returns:
                |   oShapePathName           The path name of the shape.

        """
        return self.product.GetShapePathName(i_shape_name)

    def get_shape_representation(
            self, i_load_if_necessary, i_shape_name, i_rep_behavior, i_context
    ):
        """

        i_rep_behaviour must be a string taken from KEY value of cat_rep_types

        .. note::
            CAA V5 Visual Basic help

                | GetShapeRepresentation
                | o Func GetShapeRepresentation(    boolean    iLoadIfNecessary,
                |                                   CATBSTR    iShapeName,
                |                                   CatRepType    iRepBehavior,
                |                                   boolean    iContext) As CATBaseDispatch
                |
                | Retrieves the product's  representation with the given parameters.
                |
                | Parameters:
                | iLoadIfNecessary
                |    Parameter to set to True if the master shape representation
                |    should be loaded to determine if it exists, or to False otherwise.
                |  iShapeName
                |    The name of the representation of the product.
                |  iRepBehavior
                |    The behavior of the representation.
                |    It can take the values catRep3D if the representation is a 3D one,
                |    catRep2D if the representation is a 2D one,
                |    or catRepText if the representation is a text one.
                |  iContext
                |    A condition to specify if the representation is
                |    displayed with the representation of other products.
                |
                | Examples:
                |
                | This example retrieves in MSRep the
                | Engine product's  3D representation named "PART".
                |
                | Dim MSRep As Object
                | Set MSRep = Engine.GetMasterShapeRepresentation(True,"PART",catRep3D,TRUE)

        """
        return self.product.GetShapeRepresentation(
            i_load_if_necessary, i_shape_name, cat_rep_type[i_rep_behavior], i_context
        )

    def get_technological_object(self, i_application_type):
        """

        .. note::
            CAA V5 Visual Basic help

                | GetTechnologicalObject
                | o Func GetTechnologicalObject(    CATBSTR    iApplicationType) As CATBaseDispatch
                |
                | Returns the product's applicative data which type is the given
                | parameter. The data returned can be either a collection or a simple
                | object.
                |
                | Parameters:
                | iApplicationType
                |    The type of applicative data searched.
                |
                | Examples:
                |
                | This example retrieves the constraints for the
                | Engine product.
                |
                | Dim EngineConstraints As Collection
                | Set EngineConstraints = Engine.GetTechnologicalObject("Constraints")

        """
        return self.product.GetTechnologicalObject(i_application_type)

    def has_children(self):
        """

        :return: bool
        """

        if self.product.Products.Count > 0:
            return True

        return False

    def has_shape_representation(self, i_shape_name, i_rep_behavior, i_context):
        """
        .. note::
            CAA V5 Visual Basic help

                | HasShapeRepresentation
                | o Func HasShapeRepresentation(    CATBSTR    iShapeName,
                |                                   CatRepType    iRepBehavior,
                |                                   boolean    iContext) As boolean
                |
                | Returns whether the product has a representation of the given name
                | with a given behavior. True if the product has such a representation.
                |
                | Parameters:
                | iShapeName
                |    The name of the representation of the product.
                |
                |  iRepBehavior
                |    The behavior of the representation.
                |    It can take the values catRep3D if the representation is a 3D one,
                |    catRep2D if the representation is a 2D one,
                |    or catRepText if the representation is a text one.
                |
                |  iContext
                |    A condition to specify if the representation is
                |    displayed with the representation of other products.
                |
                | Examples:
                |
                | This example retrieves in HasRep whether the
                | Engine product has a master shape representation.
                |
                | HasRep = Engine.HasRepresentation("PART",catRep3D,TRUE)

        """
        return self.product.HasShapeRepresentation(
            i_shape_name, cat_rep_type[i_rep_behavior], i_context
        )

    def is_catproduct(self):
        """

        :return: bool
        """

        if "catproduct" == self.file_name.rsplit(".")[-1].lower():
            return True

        return False

    def is_catpart(self):
        """

        :return: bool
        """

        if "catpart" == self.file_name.rsplit(".")[-1].lower():
            return True

        return False

    def remove_master_shape_representation(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | RemoveMasterShapeRepresentation
                | o Sub RemoveMasterShapeRepresentation()
                |
                | Removes the master shape representation from the product. The master
                | shape representation is the object that gives a geometric shape and
                | allows the visualization of the product. It can be a CATIA V4 model, a
                | VRML file, or any other type of document that can be displayed. In a
                | multi representation  context, the master shape representation is the
                | most  meaningful representation of the product according to the user.
                | This is the default shape for the multi representation. Note: This
                | master shape representation is optional.  Example:    This example
                | removes the master shape representation of the Engine product.
                | Engine.RemoveMasterShapeRepresentation()

        """
        return self.product.RemoveMasterShapeRepresentation()

    def remove_shape_representation(self, i_shape_name, i_rep_behavior, i_context):
        """
        .. note::
            CAA V5 Visual Basic help

                | RemoveShapeRepresentation
                | o Sub RemoveShapeRepresentation(    CATBSTR    iShapeName,
                |                                     CatRepType    iRepBehavior,
                |                                     boolean    iContext)
                |
                | Removes a specific representation from the product. A representation
                | is the object that gives a geometric shape and allows the
                | visualization of the product.. It can be a CATIA V4 model, a VRML
                | file, or any other type of document that can be displayed. Note: This
                | representation is optional.
                |
                | Parameters:
                | iShapeName
                |    The name of the representation of the product.
                |
                |  iRepBehavior
                |    The behavior of the representation.
                |    It can take the values catRep3D if the representation is a 3D one,
                |    catRep2D if the representation is a 2D one,
                |    or catRepText if the representation is a text one.
                |
                |  iContext
                |    A condition to specify if the representation is
                |    displayed with the representation of other products.
                |
                | Examples:
                |
                | This example removes the 3D representation named "PART" of the
                | Engine product.
                |
                | Engine.RemoveMasterShapeRepresentation
                | ("PART",catRep3D,TRUE)

        """
        return self.product.RemoveShapeRepresentation(
            i_shape_name, cat_rep_type[i_rep_behavior], i_context
        )

    def update(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | Update
                | o Sub Update()
                |
                | Updates the product. This update is performed with respect to the part
                | making the product or to the product's representation. It takes into
                | account the  components of the product at any level  Example:     The
                | following example updates the root product:  Dim RootProduct As
                | Product Set Rootproduct = productDoc.Product Rootproduct.Update

        """
        return self.product.Update()

    def __repr__(self):
        return f'Product(name="{self.name}")'



#! /usr/bin/python3.6

import os
import warnings

from pywintypes import com_error

from pycatia.exception_handling import CATIAApplicationException
from pycatia.in_interfaces.document import Document
from pycatia.system_interfaces.collection import Collection


class Documents(Collection):
    """
    The Documents object is used to access multiple open documents in the catia session.

    Usage::

        >>> from pycatia import catia
        >>> documents = catia.documents

    .. note::
        CAA V5 Visual Basic help

        A collection of all the Document objects currently managed by the application.
        These documents belong to one of the following types:

            PartDocument,
            ProductDocument,
            Drawing.
    """

    def __init__(self, com_object):
        super().__init__(com_object, child_object=Document)
        self.documents = com_object

    def add(self, document_type):
        """
        .. note::
            CATIA V5 Visual Basic Help

            Func Add( CATBSTR  docType) As Document

            Creates a Document object and adds it to the documents collection. This document becomes the active one,
            and a window is created to accommodate it which becomes the active window.

            | Parameters:
            |   docType
            | The type of the document to create, chosen among:
            |   Part
            |       For PartDocument
            |   Product
            |       For ProductDocument
            |   Drawing
            |       For Drawing
            |   Returns:
            |       The created document
            | Example:
            | The following example creates a PartDocument document in the collection retrieved in PartDoc.
            |   Dim PartDoc As Document
            |   Set PartDoc = Documents.Add("Part")

        :param document_type:
        :return:
        """

        document_types = ['Part', 'Product', 'Drawing']
        if document_type not in document_types:
            raise ValueError(f'Document type must be in [{document_types}]')

        self.logger.info(f'Creating a new "{document_type}".')

        return self.child_object(self.documents.Add(document_type))

    def count_types(self, file_type_list):
        """
        Returns the number of documents which presents special file extensions like:
            'catpart', 'catdrawing', 'catproduct', 'catmaterial', 'catalog', 'catfct'


        :param str (list) file_type_list: filetype(s) to count.
        :return: int()
        """

        items = self.get_item_names()

        if not type(file_type_list) == list:
            file_type_list = [elem.lower() for elem in [file_type_list]]
        else:
            file_type_list = [elem.lower() for elem in file_type_list]

        return len([True for name in items for typ in file_type_list if name.lower().find(typ) > 0])

    def new_from(self, file_name):
        """

        .. note::
            CAA V5 Visual Basic help

            Creates a new document from a document stored in a file. Role: Reads a document stored in a file and
            creates a new document containing the resulting data, adds the new document to the document collection,
            displays it in a new window, adds the window to the window collection and activates both the document and
            the window.

            | Parameters:
            |   The name of the file containing the document.
            | Returns:
            |   The created document.
            | Example:
            | The following example creates a new Doc document from the contents of the FileToRead file.
            |   FileToRead = "e:////users////psr////Parts////ThisIsANicePart.CATPart"
            |   Dim Doc As Document
            |   Set Doc = Documents.NewFrom(FileToRead)


        :param str file_name:
        :return:

        """

        if not os.path.isfile(file_name):
            raise FileNotFoundError(f'Could not find file {file_name}.')

        # get the full path to the file
        file_name = os.path.abspath(file_name)

        return self.documents.NewFrom(file_name)

    def item(self, index):
        """
        .. note::
            CAA V5 Visual Basic help

                | Item
                | o Func Item(CATVariant iIndex) As Document
                |
                | Returns a document using its index or its name from the documents
                | collection.


                | Parameters:
                | iIndex
                |    The index or the name of the document to retrieve frm
                |    the collection of documents.
                |    As a numeric, this index is the rank of the document
                |    in the collection.
                |    The index of the first document in the collection is 1, and
                |    the index of the last document is Count.
                |    As a string, it is the name you assigned to the document using
                |    the
                |
                |  activateLinkAnchor('AnyObject','Name','AnyObject.Name')  property.
                |    Returns:
                |   The retrieved document


                | Examples:
                |
                |
                | This example retrieves in ThisDoc the fifth document
                | in the collection and in ThatDoc the document
                | named MyDoc.
                |
                | Dim ThisDoc As Document
                | Set ThisDoc = Documents.Item(5)
                | Dim ThatDoc As Document
                | Set ThatDoc = Documents.Item("MyDoc")
                |

        :param int|str index:
        :return Document COM object:
        """

        return self.child_object(self.documents.Item(index))

    def num_open(self):
        """
        Returns the number of open documents.

        .. warning::

            The COM object can return the incorrect number of documents open. After a document is closed CATIA can keep
            the linked document `ABQMaterialPropertiesCatalog.CATfct` open.

        :return: int()
        """

        # for i in range(0, self.documents.Count):
        #     print(self.documents.Item(i + 1).Name)

        warning_string = (
            'The COM object can return the incorrect number of documents open. //n'
            'For example, after a CATPart document is closed CATIA can keep'
            'the linked document `ABQMaterialPropertiesCatalog.CATfct` loaded in the session.'
        )

        warnings.warn(warning_string)
        return self.documents.Count

    def open(self, file_name):
        """
        Open CATIA document `file_name` in current CATIA session.

        .. note::
            CAA V5 Visual Basic help

            Func Open( CATBSTR  iFileName) As Document

            Opens a document stored in a file. Reads a document stored in a file, displays it in a new window, adds the
            document to the documents collection and the window to the windows collection, and makes both the document
            and the window the active ones.

            | Parameters:
            |   iFileName:
            |       The name of the file containing the document
            |   Returns:
            |       The retrieved document

            | Example:
            | The following example opens the Doc document contained in the FileToOpen file.
            | FileToOpen = "e:/users/psr/Parts/ThisIsANicePart.CATPart"
            | Dim Doc As Document
            | Set Doc = Documents.Open(FileToOpen)

        :param str file_name: full path to catia file.
        """

        if not os.path.isfile(file_name):
            raise FileNotFoundError(f'Could not find file {file_name}.')

        # get the full path to the file
        file_name = os.path.abspath(file_name)

        try:
            self.logger.info(f'Opening document "{file_name}".')
            self.documents.Open(file_name)
        except com_error:
            raise CATIAApplicationException(
                f'Could not open document "{file_name}". '
                'Check file type and ensure the version of CATIA it was created with is compatible.')

    def read(self, file_name):
        """
        .. note::
            CAA V5 Visual Basic help

                | Read
                | o Func Read(    CATBSTR    iFileName) As Document
                |
                | Reads a document stored in a file. This method has to be used only for
                | Browse purpose, for instance to retrieve Product properties. Be
                | careful, it doesn't open any editor (no visualization, no undo/redo
                | capabilities...) As soon as you want to modify the V5 document, you
                | have to use  the VB Open method collection. If this solution is not
                | satisfactory because it opens an editor for every document, you have
                | to move to C++ and use the CAA methods CATDocumentServices::Open and
                | CATDocumentServices::SaveAs with the same file name as the initial
                | one.
                |
                | Parameters:
                | iFileName
                |    The name of the file containing the document
                |
                |
                |  Returns:
                |   The retrieved document
                |
                | Examples:
                |
                |
                | The following example reads the Doc document
                | contained in the FileToOpen file.
                |
                | FileToOpen = "e://users//psr//Parts//ThisIsANicePart.CATPart"
                | Dim Doc As Document
                | Set Doc = Documents.Read(FileToOpen)
                |
                |
                |
        """
        return self.documents.Read(file_name)

    def __repr__(self):
        return f'Documents(name="{self.name}")'



#! /usr/bin/python3.6
# module initially auto generated using V5Automation.chm from CATIA V5 R25


class Move:
    """
        .. note::
            CAA V5 Visual Basic help

                | Represents an object associated with any kind of objects which
                | retrieves the corresponding movable object.An object can be moved only
                | if it is independent from other objects. For example, a pad can be
                | moved, and is as such a movable object, while an edge cannot be moved
                | individually. It should be moved with all the objects making up the
                | pad to which it belongs as a whole. In this case, the underlying
                | edge's movable object is the pad.

    """

    def __init__(self, movable):
        self.move = movable.Move

    @property
    def movable_object(self):
        """
        .. note::
            CAA V5 Visual Basic help

                | MovableObject
                | o Property MovableObject() As Move
                |
                | Returns the movable object associated with the used object. Example:
                | This example retrieves the myMovableObject from  Move object
                | associated with MyObject.  Dim myMovableObject As Move Set
                | myMovableObject = MyObject.Move.MovableObject


                | Parameters:


        """
        return self.move.MovableObject

    def apply(self, i_transformation_array):
        """
        .. note::
            CAA V5 Visual Basic help

                | Apply
                | o Sub Apply(    CATSafeArrayVariant    iTransformationArray)
                |
                | Applies a move transformation to a movable object.


                | Parameters:
                | TransformationArray
                |    The linear transformation array initialized successively by the
                |    four columns of the transformation matrix.
                |    The first nine components represent the rotation matrix.
                |    The last three components represent the translation vector.


                | Examples:
                |
                | This example applies the transformation (45 degrees-rotation around
                | the x axis and a translation) stored in TransformationArray
                | to the Move object associated with myMovableObject:
                |
                | Dim TransformationArray( 11 )
                | 'Rotation( 45 degrees around the x axis) components
                | TransformationArray( 0 )  = 1.000
                | TransformationArray( 1 )  = 0
                | TransformationArray( 2 )  = 0
                | TransformationArray( 3 )  = 0
                | TransformationArray( 4 )  = 0.707
                | TransformationArray( 5 )  = 0.707
                | TransformationArray( 6 )  = 0
                | TransformationArray( 7 )  = -0.707
                | TransformationArray( 8 )  = 0.707
                | 'Translation Vector (10,20,30)
                | TransformationArray( 9 )  = 10.000
                | TransformationArray( 10 ) = 20.000
                | TransformationArray( 11 ) = 30.000
                | MyMovableObject.Move.Apply TransformationArray
                |
        """

        return self.move.Apply(i_transformation_array)
